==> 3d-tiles-tools-0.2.0.tgz: package/src/spatial/OctreeCoordinates.ts <==
import { MortonOrder } from "./MortonOrder";
import { Octrees } from "./Octrees";
import { TreeCoordinates } from "./TreeCoordinates";

/**
 * An implementation of `TreeCoordinates` for octrees
 *
 * @internal
 */
export class OctreeCoordinates implements TreeCoordinates {
  private readonly _level: number;
  private readonly _x: number;
  private readonly _y: number;
  private readonly _z: number;

  constructor(level: number, x: number, y: number, z: number) {
    this._level = level;
    this._x = x;
    this._y = y;
    this._z = z;
  }

  /** {@inheritDoc TreeCoordinates.level} */
  get level(): number {
    return this._level;
  }

  get x(): number {
    return this._x;
  }

  get y(): number {
    return this._y;
  }

  get z(): number {
    return this._z;
  }

  /** {@inheritDoc TreeCoordinates.parent} */
  parent(): OctreeCoordinates | null {
    if (this._level === 0) {
      return null;
    }
    const pLevel = this._level - 1;
    const px = this._x >> 1;
    const py = this._y >> 1;
    const pz = this._z >> 1;
    return new OctreeCoordinates(pLevel, px, py, pz);
  }

  /** {@inheritDoc TreeCoordinates.children} */
  *children(): IterableIterator<OctreeCoordinates> {
    const nLevel = this._level + 1;
    const nX = this._x << 1;
    const nY = this._y << 1;
    const nZ = this._z << 1;
    yield new OctreeCoordinates(nLevel, nX + 0, nY + 0, nZ + 0);
    yield new OctreeCoordinates(nLevel, nX + 1, nY + 0, nZ + 0);
    yield new OctreeCoordinates(nLevel, nX + 0, nY + 1, nZ + 0);
    yield new OctreeCoordinates(nLevel, nX + 1, nY + 1, nZ + 0);
    yield new OctreeCoordinates(nLevel, nX + 0, nY + 0, nZ + 1);
    yield new OctreeCoordinates(nLevel, nX + 1, nY + 0, nZ + 1);
    yield new OctreeCoordinates(nLevel, nX + 0, nY + 1, nZ + 1);
    yield new OctreeCoordinates(nLevel, nX + 1, nY + 1, nZ + 1);
  }

  /** {@inheritDoc TreeCoordinates.descendants} */
  descendants(
    maxLevelInclusive: number,
    depthFirst: boolean
  ): IterableIterator<OctreeCoordinates> {
    const queue: OctreeCoordinates[] = [this];
    const result = {
      [Symbol.iterator]() {
        return this;
      },
      next(): IteratorResult<OctreeCoordinates, void> {
        const element = depthFirst ? queue.pop() : queue.shift();
        if (!element) {
          return { value: undefined, done: true };
        }
        if (element.level < maxLevelInclusive) {
          for (const c of element.children()) {
            queue.push(c);
          }
        }
        return { value: element, done: false };
      },
    };
    return result;
  }

  /** {@inheritDoc TreeCoordinates.toArray} */
  toArray(): number[] {
    return [this.level, this.x, this.y, this.z];
  }

  /** {@inheritDoc TreeCoordinates.toIndex} */
  toIndex(): number {
    const offset = Octrees.computeNumberOfNodesForLevels(this._level);
    return offset + this.toIndexInLevel();
  }

  /** {@inheritDoc TreeCoordinates.toIndexInLevel} */
  toIndexInLevel(): number {
    return MortonOrder.encode3D(this._x, this._y, this._z);
  }

  toString = (): string => {
    return `(level ${this.level}, (${this._x},${this._y},${this._z}))`;
  };
}

==> 3d-tiles-tools-0.3.2.tgz: package/src/spatial/OctreeCoordinates.ts <==
import { MortonOrder } from "./MortonOrder";
import { Octrees } from "./Octrees";
import { TreeCoordinates } from "./TreeCoordinates";

/**
 * An implementation of `TreeCoordinates` for octrees
 *
 * @internal
 */
export class OctreeCoordinates implements TreeCoordinates {
  private readonly _level: number;
  private readonly _x: number;
  private readonly _y: number;
  private readonly _z: number;

  constructor(level: number, x: number, y: number, z: number) {
    this._level = level;
    this._x = x;
    this._y = y;
    this._z = z;
  }

  /** {@inheritDoc TreeCoordinates.level} */
  get level(): number {
    return this._level;
  }

  get x(): number {
    return this._x;
  }

  get y(): number {
    return this._y;
  }

  get z(): number {
    return this._z;
  }

  /** {@inheritDoc TreeCoordinates.parent} */
  parent(): OctreeCoordinates | null {
    if (this._level === 0) {
      return null;
    }
    const pLevel = this._level - 1;
    const px = this._x >> 1;
    const py = this._y >> 1;
    const pz = this._z >> 1;
    return new OctreeCoordinates(pLevel, px, py, pz);
  }

  /** {@inheritDoc TreeCoordinates.children} */
  children(): Iterable<OctreeCoordinates> {
    const nLevel = this._level + 1;
    const nX = this._x << 1;
    const nY = this._y << 1;
    const nZ = this._z << 1;
    const iterable = {
      [Symbol.iterator]: function* (): Iterator<OctreeCoordinates> {
        yield new OctreeCoordinates(nLevel, nX + 0, nY + 0, nZ + 0);
        yield new OctreeCoordinates(nLevel, nX + 1, nY + 0, nZ + 0);
        yield new OctreeCoordinates(nLevel, nX + 0, nY + 1, nZ + 0);
        yield new OctreeCoordinates(nLevel, nX + 1, nY + 1, nZ + 0);
        yield new OctreeCoordinates(nLevel, nX + 0, nY + 0, nZ + 1);
        yield new OctreeCoordinates(nLevel, nX + 1, nY + 0, nZ + 1);
        yield new OctreeCoordinates(nLevel, nX + 0, nY + 1, nZ + 1);
        yield new OctreeCoordinates(nLevel, nX + 1, nY + 1, nZ + 1);
      },
    };
    return iterable;
  }

  /** {@inheritDoc TreeCoordinates.descendants} */
  descendants(
    maxLevelInclusive: number,
    depthFirst: boolean
  ): Iterable<OctreeCoordinates> {
    // eslint-disable-next-line @typescript-eslint/no-this-alias
    const root = this;
    const resultIterable = {
      [Symbol.iterator]: function* (): Iterator<OctreeCoordinates> {
        const queue: OctreeCoordinates[] = [root];
        while (true) {
          const element = depthFirst ? queue.pop() : queue.shift();
          if (!element) {
            break;
          }
          if (element.level < maxLevelInclusive) {
            for (const c of element.children()) {
              queue.push(c);
            }
          }
          yield element;
        }
      },
    };
    return resultIterable;
  }

  /** {@inheritDoc TreeCoordinates.toArray} */
  toArray(): number[] {
    return [this.level, this.x, this.y, this.z];
  }

  /** {@inheritDoc TreeCoordinates.toIndex} */
  toIndex(): number {
    const offset = Octrees.computeNumberOfNodesForLevels(this._level);
    return offset + this.toIndexInLevel();
  }

  /** {@inheritDoc TreeCoordinates.toIndexInLevel} */
  toIndexInLevel(): number {
    return MortonOrder.encode3D(this._x, this._y, this._z);
  }

  toString = (): string => {
    return `(level ${this.level}, (${this._x},${this._y},${this._z}))`;
  };
}
